#!/usr/bin/env python3
from google.auth.transport import requests
from google.oauth2 import id_token


def generatePolicy(principal: str, idInformation: {}, effect: str, methodArn) -> {}:
    """
        Generates a policy based off of the user's information.
    """
    authResponse: {} = {}
    authResponse['principalId'] = principal

    if effect and methodArn:
        authResponse['policyDocument'] = {
            'Version': '2012-10-17',
            'Statement': [
                {
                    'Sid': 'FirstStatement',
                    'Action': 'execute-api:Invoke',
                    'Effect': effect,
                    'Resource': methodArn
                }
            ]
        }

        context: {} = {}
        if 'name' in idInformation:
            context['name'] = idInformation['name']

        if 'picture' in idInformation:
            context['picture'] = idInformation['picture']

        if 'locale' in idInformation:
            context['locale'] = idInformation['locale']

        authResponse['context'] = context

    return authResponse


def verify(token: str, clientID: str) -> {}:
    idInformation: {}

    try:
        # Verify and get information from id_token.
        idInformation = id_token.verify_oauth2_token(
            token,
            requests.Request(),
            f'{clientID}.apps.googleusercontent.com'
        )

        # Deny access if the account is not a Google account.
        if idInformation['iss'] not in [
            'accounts.google.com',
            'https://accounts.google.com'
        ]:
            return None

    except ValueError as e:
        print(e)

        # Deny access if the token is invalid
        return None

    return idInformation



import json
import requests
from urllib.parse import quote

class DiscordAuth:
    def __init__(
        self,
        discord_api: str,
        redirect_url: str,
        client_id: str
    ):
        super()
        self.redirect_url = quote(redirect_url, safe='')
        self.oath_api = f'{discord_api}/oauth2'
        self.client_id = client_id


    def request_authorization(
        self,
        state: str,
        scope: str = 'identity',
        prompt: str = 'none'
    ) -> str:
        """
            Request a URL the user can use to request authorization
            from Discord.
        """
        if not state:
            raise ValueError('A state is required.')

        auth_url = f'{self.oath_api}/authorize?response_type=code' + \
            f'&client_id={self.client_id}&state={state}&scope={scope}' + \
            f'&redirect_uri={self.redirect_url}&prompt={prompt}'

        return auth_url


    def request_access(
        self,
        code: str,
        client_secret: str
    ):
        """
            Get the access token for a proper authorization.
        """

        if not code:
            raise ValueError('Auth code required.')

        if not client_secret:
            raise ValueError('Client secret required.')

        data = {
            'client_id': self.client_id,
            'client_secret': client_secret,
            'grant_type': 'authorization_code',
            'code': code,
            'redirect_uri': self.redirect_url
        }
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded'
        }
        response = requests.post(
            f'{self.oath_api}/token',
            data=data,
            headers=headers
        )

        # Check for errors.
        response.raise_for_status()

        tokens = json.loads(response.content)
        return tokens['access_token']


    def get_user(
        self,
        access_token: str
    ):
        """
            Pings Discord's API to get the user of the access token.
        """
        return json.loads(
            requests.get(
                f'{self}/@me',
                headers={
                    'Authorization': f'Bearer {access_token}'
                }
            ).content
        )['user']


import boto3
import botocore


class S3Bucket:
    def __init__(self, bucketName, region, endpoint=None):
        s3 = boto3.resource('s3', region_name=region, endpoint_url=endpoint)
        self._bucket = s3.Bucket(bucketName)

    def download(self, objectKey, targetFileName=None):
        if targetFileName is None:
            targetFileName = objectKey

        try:
            self._bucket.download_file(objectKey, targetFileName)
        except botocore.exceptions.ClientError as e:
            if e.response['Error']['Code'] == "404":
                print("The object does not exist.")
            else:
                raise

    def upload(self, filename, objectKey=None):
        if objectKey is None:
            objectKey = filename

        try:
            self._bucket.upload_file(filename, objectKey)
        except botocore.exceptions.ClientError as e:
            if e.response['Error']['Code'] == "404":
                print("Could not upload the file.")
            else:
                raise

    def delete(self, objectKey):
        try:
            self._bucket.delete_object(objectKey)
        except botocore.exceptions.ClientError as e:
            if e.response['Error']['Code'] == "404":
                print("The object does not exist.")
            else:
                raise



#!/bin/python
import time

def from_seconds(seconds: float) -> time:
    return time.gmtime(seconds)


def parse_timestamp(timestamp: str) -> time:
    try:
        return time.strptime(timestamp, '%H:%M:%S')
    except ValueError:
        return time.strptime(timestamp, '%M:%S')


def stringify(timestamp: time) -> str:
    return time.strftime('%H:%M:%S', timestamp)


def duration(end: time, start: time) -> time:
    return from_seconds(
        time.mktime(end)
        - time.mktime(start)
    )


import boto3
from botocore.exceptions import ClientError


class DynamoTable:
    def __init__(self, name, region, endpoint=None):

        # Get a DynamoDB client to use for this region.
        dynamo = boto3.resource(
            'dynamodb',
            region_name=region,
            endpoint_url=endpoint
        )

        # Attempt to access the table.
        self._table = dynamo.Table(name)

    def get_item(self, key: {}):
        return self._table.get_item(Key=key)

    def query(self, expression):
        return self._table.query(KeyConditionExpression=expression)

    def batch_get_item(self, keys: []):
        try:
            return self._table.batch_get_item(
                Keys=keys
            )['Responses']
        except ClientError as e:
            print(e.response['Error']['Message'])

    def try_get_item(self, key: {}) -> {}:

        try:
            # Lookup the object.
            response = self.get_item(key=key)

            # Check if there is a response to return.
            if not response or 'Item' not in response or \
                    not response['Item']:
                return None
            else:
                return response['Item']
        except ClientError:
            return None

    def put_item(self, item: {}) -> {}:
        return self._table.put_item(Item=item)

    def delete_item(self, key: {}) -> {}:
        return self._table.delete_item(Key=key)



import boto3
from base64 import b64decode


class Kms:
    def __init__(self, region: str):

        # Create and store a kms client.
        self.kms = boto3.client('kms', region_name=region)

    def decrypt(self, cypherText: str) -> str:
        return self.kms.decrypt(
            CiphertextBlob=b64decode(cypherText)
        )['Plaintext']

    def encrypt(self, keyID: str, plaintext: str) -> str:
        return self.kms.encrypt(
            KeyId=keyID,
            Plaintext=b64decode(plaintext)
        )['CiphertextBlob']



#!/usr/bin/env python3
from uuid import UUID
import re


def createResponse(statusCode: int, body: {}):
    """
        Creates a response object with a user provided status code and message.
    """
    return {
        # 'isBase64Encoded': True,
        'statusCode': statusCode,
        # 'headers': {},
        'body': body
    }


def createMessageResponse(statusCode: int, message: str):
    """
        Creates a response with a formatted error message.
    """
    return createResponse(statusCode, {
        'message': message
    })


def success(body: {}):
    """
        Creates a response object with a
        '200' status code and a payload.
    """
    return createResponse(200, body)


def accepted(message: str):
    """
        Creates a response object with a
        '200' status code and the user's message.
    """
    return createMessageResponse(202, message)


def serverError(message: str):
    """
        Creates a response object with a
        '500' status code and the user's message.
    """
    return createMessageResponse(500, message)


def badRequest(message: str):
    """
        Creates a response object with a
        '400' status code and the user's message.
    """
    return createMessageResponse(400, message)


def forbidden(message: str):
    """
        Creates a response object with a
        '403' status code and the user's message.
    """
    return createMessageResponse(403, message)


def unauthorized(message: str):
    """
        Creates a response object with a
        '401' status code and the user's message.
    """
    return createMessageResponse(401, message)


def isValidUuid(string: str, version: int = 4) -> bool:
    """
    Check if string is a valid UUID.

    Args:
        string : str
        version : {1, 2, 3, 4}
    """
    try:
        string = string.lower()
        uuidObj = UUID(string, version=version)
    except ValueError:
        return False

    return str(uuidObj).lower() == string


def isUrl(string: str) -> bool:
    """
        Check if a string is a valid URL.
    """

    # https://stackoverflow.com/a/7160778/10167844
    regex = re.compile(
        r'^(?:(?:http|ftp)s?://|)'
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+'
        r'(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'
        r'localhost|'
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'
        r'(?::\d+)?'
        r'(?:/?|[/?]\S+)$', re.IGNORECASE)

    return re.match(regex, string) is not None


def isIP(string: str) -> bool:
    """
        Check if a string is a valid IP address.
    """
    regex = re.compile(r"^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$")
    return re.match(regex, string) is not None
